/*	$NetBSD$	*/

/*
 * Copyright (c) 2010 SHIMIZU Ryo <ryo@misakimix.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _AICAREG_H_
#define _AICAREG_H_

#define AICA_REG_ADDR_ARM	0x00800000
#define AICA_REG_ADDR_SH4	0x00700000
#define AICA_REG_SIZE		0x00003000

#define AICA_RAM_START_ARM	0x00000000
#define AICA_RAM_START_SH4	0x00800000
#define AICA_RAM_SIZE		0x00200000


/* 0x00800000(AICA) 0xa0700000(SH4 G2 FIFO) */
#define CH_ADDR(ch, addr)		((0x80 * (ch)) + (addr))

#define AICA_PLAYCTRL_REG(ch)		CH_ADDR(ch, 0x00000000)
#define AICA_PLAYCTRL_KEYON			0xc000
#define AICA_PLAYCTRL_KEYOFF			0x8000
#define AICA_PLAYCTRL_NOISE			0x0400
#define AICA_PLAYCTRL_LOOP			0x0200
#define AICA_PLAYCTRL_PCM16BIT			0x0000
#define AICA_PLAYCTRL_PCM8BIT			0x0080
#define AICA_PLAYCTRL_ADPCM			0x0100
#define AICA_PLAYCTRL_LONGADPCM			0x0180
#define AICA_PLAYCTRL_ADDRHI(addr)		(((addr) >> 16) & 0x7f)
#define AICA_ADDRLO_REG(ch)		CH_ADDR(ch, 0x00000004)
#define AICA_ADDRLO_ADDR(addr)			((addr) & 0xffff)
#define AICA_LOOP_START_REG(ch)		CH_ADDR(ch, 0x00000008)	/* 0000-fffd */
#define AICA_LOOP_END_REG(ch)		CH_ADDR(ch, 0x0000000c)	/* 0001-ffff */
#define AICA_ENVELOPE1_REG(ch)		CH_ADDR(ch, 0x00000010)
#define AICA_ENVELOPE2_REG(ch)		CH_ADDR(ch, 0x00000014)
#define AICA_RATE_REG(ch)		CH_ADDR(ch, 0x00000018)
#define AICA_LFO1_REG(ch)		CH_ADDR(ch, 0x0000001c)
#define AICA_DSPCFG_REG(ch)		CH_ADDR(ch, 0x00000020)
#define AICA_VOLPAN_REG(ch)		CH_ADDR(ch, 0x00000024)
#define AICA_VOLPAN_VOL(v)			((v) << 8)
#define AICA_VOLPAN_PAN(v)			((v) & 0x1f)

#define AICA_TL_Q_REG(ch)		CH_ADDR(ch, 0x00000028)
#define AICA_FLV0_REG(ch)		CH_ADDR(ch, 0x0000002c)
#define AICA_FLV1_REG(ch)		CH_ADDR(ch, 0x00000030)
#define AICA_FLV2_REG(ch)		CH_ADDR(ch, 0x00000034)
#define AICA_FLV3_REG(ch)		CH_ADDR(ch, 0x00000038)
#define AICA_FLV4_REG(ch)		CH_ADDR(ch, 0x0000003c)
#define AICA_AR_D1R_REG(ch)		CH_ADDR(ch, 0x00000040)
#define AICA_D2R_RR_REG(ch)		CH_ADDR(ch, 0x00000044)

#define AICA_VOLUME_REG			0x00002800
#define AICA_DSPADDR_REG		0x00002804
#define AICA_BUSREQ_REG			0x00002808
#define AICA_BUSREQ_MASK			0x0100

#define AICA_CHANSEL_REG		0x0000280c
#define AICA_CHANSEL_CH(ch)			((ch) << 8)
#define AICA_CHANSTAT_REG		0x00002810
#define AICA_CHANPOS_REG		0x00002814

#define AICA_TIMER1CTRL_REG		0x00002890
#define AICA_TIMER2CTRL_REG		0x00002894
#define AICA_TIMER3CTRL_REG		0x00002898

#define AICA_TIMEREND_REG		0x0000289c
#define AICA_TIMERCTRL_REG		0x000028a4
#define AICA_TIMERCTRL_ENABLE			0x0040
#define AICA_TIMERCTRL_DISABLE			0x0000

/* ARM CPU Clock */
#define AICA_CLOCK_REG			0x000028a8
#define AICA_RSVD1_REG			0x000028ac
#define AICA_RSVD2_REG			0x000028b0

/* interrupt */
#define AICA_INTLVL0_REG		0x000028a8
#define AICA_INTLVL1_REG		0x000028ac
#define AICA_INTLVL2_REG		0x000028b0

#define AICA_INTEN_REG			0x000028b4
#define AICA_INTSET_REG			0x000028b8
#define AICA_INTRST_REG			0x000028bc
#define AICA_INT_CPU				0x0020
#define AICA_INT_TIMERA				0x0040
#define AICA_INT_TIMERB				0x0080
#define AICA_INT_TIMERC				0x0100
#define AICA_INT_SAMPLE				0x0400

/* ARM Reset */
#define AICA_RESETCTL_REG		0x00002c00
#define AICA_RESETCTL_CABLE_MASK		0x0030
#define AICA_RESETCTL_CABLE_VGA			0x0000	/* ro */
#define AICA_RESETCTL_CABLE_RGB			0x0020	/* ro */
#define AICA_RESETCTL_CABLE_CONPOSITE		0x0030	/* ro */
#define AICA_RESETCTL_RESET			0x0001	/* rw */

/* ARM interrupt control */
#define AICA_ARMINTNUM_REG		0x00002d00
#define AICA_ARMINTRST_REG		0x00002d04
#define AICA_ARMINTRST_WRITABLE			0x00000000
#define AICA_ARMINTRST				0x00000001

#endif /* _AICAREG_H_ */
