/*	$NetBSD: aicavar.h,v 1.3 2005/12/11 12:17:06 christos Exp $	*/

/*
 * Copyright (c) 2003 SHIMIZU Ryo <ryo@misakimix.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _AICAVAR_H_
#define _AICAVAR_H_

/*
 *	AICA ARM/SH4 Memory usage MAP
 *
 *	ARM7 side :                                 : SH4 side
 *	----------: ------------------------------- :-----------
 *	0x00000000: arm code (microcode/aica_arm.c) : 0x00800000
 *	          : arm data, bss, stack            :
 *	0x00008000: ARM to SH4 command buffer       : 0x00808000
 *	0x0000C000: SH4 to ARM command buffer       : 0x0080C000
 *	0x00010000: PCM BUFFER                      : 0x00810000
 *	0x001FFFFF: end of RAM                      : 0x009FFFFF
 *
 *	0x00800000: AICA Registers                  : 0x00700000
 *	0x00802FFF:                                 : 0x00702FFF
 */

#define AICA_ARM_CODE		0x00000000	/* text+data+bss+stack */

#define AICA_MSGBUF		0x0000e000
#define AICA_MSGBUFSIZE		0x00001000

#define AICA_ARMtoSH4_CMDQBUF	0x0000f000
#define AICA_ARMtoSH4_CMDQSIZE	0x00000800
#define AICA_SH4toARM_CMDQBUF	0x0000f800
#define AICA_SH4toARM_CMDQSIZE	0x00000700

#define AICA_HEARTBEAT		0x0000ff00
#define AICA_DEBUGPORT1		0x0000ff04
#define AICA_DEBUGPORT2		0x0000ff08
#define AICA_DEBUGPORT3		0x0000ff0c

#define AICA_DMABUF(unit)	(0x00010000 + 0x00010000 * 2 * (unit))
#define AICA_DMABUF_LEFT(unit)	AICA_DMABUF(unit)
#define AICA_DMABUF_RIGHT(unit)	(AICA_DMABUF(unit) + 0x00010000)
#define AICA_DMABUF_SIZE	0x0000fff0

#define LEVEL256TO16(l)	(((l) >> 4) & 0x0f)
#define LEVEL16TO256(l)	((((l) << 4) & 0xf0) + ((l) & 0x0f))

enum MIXER_CLASS {
	AICA_MASTER_VOL = 0,
	AICA_OUTPUT_GAIN,
	AICA_OUTPUT_CLASS,
	AICA_NDEVS
};

struct cmdqueue {
	uint32_t address;
	uint32_t size;
	uint32_t index;
	uint32_t serial;
};

struct cmd {
	uint32_t command;
#define AICA_COMMAND_PACK(cmd, unit)	(((unit) << 16) | (cmd))
#define AICA_COMMAND_UNIT(cmd)		((cmd) >> 16)
#define AICA_COMMAND_CMD(cmd)		((cmd) & 0xffff)

#define AICA_CMD_NOP			0x0000	/* SH4 -> AICA */
#define AICA_CMD_MSGBUF			0x0001
#define AICA_CMD_FORWARD		0x0002
#define AICA_CMD_PLAY			0x8000	/* AICA -> SH4 */
#define AICA_CMD_STOP			0x8001
#define AICA_CMD_CH_RATE		0x8002
#define AICA_CMD_PREC_BLKSIZE		0x8003
#define AICA_CMD_NBLK_NSMPL		0x8004
	uint32_t param;
#define AICA_PARAM_PACK(hi, lo)		(((hi) << 16) | (lo))
#define AICA_PARAM_HI(v)		((v) >> 16)
#define AICA_PARAM_LO(v)		((v) & 0xffff)
	uint32_t serial;
	uint32_t magic;
#define CMDMAGIC 0x41494341	/* 'AICA' */
};

#endif /* !_AICAVAR_H_ */
