/*	$NetBSD: aica_arm.c,v 1.4 2005/12/24 20:06:59 perry Exp $	*/

/*
 * Copyright (c) 2003 SHIMIZU Ryo <ryo@misakimix.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "aica_arm.h"
#include <arch/dreamcast/dev/g2/aicavar.h>

#define	MSGBUF		((char *)AICA_MSGBUF)
#define	MSGBUFSIZE	AICA_MSGBUFSIZE

static void msg_putc(char);
static int msg_puts(char *);

static int msgbuf_ptr;

int
aica_msgbuf_update()
{
	static int lastptr;

	if (lastptr == msgbuf_ptr)
		return 0;

	lastptr = msgbuf_ptr;
	return 1;
}

static void
msg_putc(char c)
{
	MSGBUF[msgbuf_ptr++] = c;
	if (msgbuf_ptr >= MSGBUFSIZE)
		msgbuf_ptr = 0;
	MSGBUF[msgbuf_ptr] = '\0';
}

static int
msg_puts(char *s)
{
	int n;

	for (n = 0; *s; ) {
		msg_putc(*s++);
		n++;
	}

	return n;
}

static const char hextable_l[] = "0123456789abcdef";
static const char hextable_u[] = "0123456789ABCDEF";

int
aica_vprintf(const char *fmt, va_list ap)
{
#define MAXPADDING	128
	char padbuf[MAXPADDING];
	char *tp;
	unsigned int base, i;
	int padding, rc;
	char c, padchr;

	rc = 0;

	for (;;) {
 nextchr:
		c = *fmt++;
		if (c == '\0')
			break;

		if (c != '%') {
			msg_putc(c);
			rc++;
			continue;
		}

		/* start '%' */
		padding = 0;
		padchr = ' ';

		while ((c = *fmt++) != '\0') {
			switch (c) {
			case '%':
				msg_putc(c);
				goto nextchr;
			case '0':
				if (padding == 0)
					padchr = '0';
				/* FALLTHRU */
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':
				padding = padding * 10 + (c - '0');
				if (padding > MAXPADDING - 1)
					padding = MAXPADDING - 1;
				break;
			case 'X':
			case 'd':
			case 'x':
				{
					const char *tbl;
					int len;

					base = (c == 'd') ? 10 : 16;
					tbl = (c == 'x') ? hextable_l : hextable_u;

					i = va_arg(ap, int);
					tp = &padbuf[MAXPADDING];
					*--tp = '\0';
					while (i >= base) {
						*--tp = tbl[i % base];
						i /= base;
					}
					*--tp = tbl[i];

					len = &padbuf[MAXPADDING - 1] - tp;
					if (padding > len)
						for (padding -= len; padding > 0; padding--)
							*--tp = padchr;

					rc += msg_puts(tp);
				}
				goto nextchr;
			default:
				goto nextchr;
			}
		}
	}
	return rc;
}

int
aica_printf(const char *fmt, ...)
{
	int rc;
	va_list ap;

	va_start(ap, fmt);
	rc = aica_vprintf(fmt, ap);
	va_end(ap);
	return rc;
}
